// Copyright 2005-2024 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// STL container functions.

#ifndef THRAX_THRAX_COMPAT_STLFUNCTIONS_H_
#define THRAX_THRAX_COMPAT_STLFUNCTIONS_H_

namespace thrax {

template <class ForwardIterator>
void STLDeleteContainerPointers(ForwardIterator begin,
                                ForwardIterator end) {
  while (begin != end) {
    ForwardIterator temp = begin;
    ++begin;
    delete *temp;
  }
}

template <class T>
void STLDeleteElements(T *container) {
  if (!container) return;
  STLDeleteContainerPointers(container->begin(), container->end());
  container->clear();
}

template <class Collection>
typename Collection::value_type::second_type *const
InsertOrReturnExisting(Collection *const collection,
                       const typename Collection::value_type &vt) {
  const auto ret = collection->insert(vt);
  if (ret.second) {
    return nullptr;  // Inserted, no existing previous value.
  } else {
    return &ret.first->second;  // Return address of already existing value.
  }
}

// Same as above, except for explicit key and data.
template <class Collection>
typename Collection::value_type::second_type *const
InsertOrReturnExisting(Collection *const collection,
    const typename Collection::value_type::first_type &key,
    const typename Collection::value_type::second_type &data) {
  return InsertOrReturnExisting(collection, {key, data});
}

}  // namespace thrax

#endif  // THRAX_THRAX_COMPAT_STLFUNCTIONS_H_
